import IconSvg from "./IconSvg.js";
import Preferences from "./Preferences.js";

export default class ItemHTML {

    static SUCCESS_CLASS_NAME = 'success';
    static WARNING_CLASS_NAME = 'warning';
    static ERROR_CLASS_NAME = 'error';

    static CONTAINER_CLASS_NAME = 'midefos-idealista-container';
    static INFORMATION_CONTAINER_CLASS_NAME = 'midefos-idealista-information-container';
    static get CONTAINER_SELECTOR() {
        return `.${this.CONTAINER_CLASS_NAME}`;
    }

    static INFORMATION_CLASS_NAME = 'information'
    static get INFORMATION_SELECTOR() {
        return `.${this.INFORMATION_CLASS_NAME}`;
    }

    static ITEM_STATE_CLASS_NAME(item) {
        const desiredPrice = Preferences.get('max-price');
        if (desiredPrice && !item.isProperPrice(desiredPrice)) {
            return this.ERROR_CLASS_NAME;
        }

        const desiredPricePerMeter = Preferences.get('max-price-per-meter');
        if (desiredPricePerMeter && !item.isProperPriceMeter(desiredPricePerMeter)) {
            return this.ERROR_CLASS_NAME;
        }

        if (this._shouldCheckGarage() && !item.hasGarage) {
            return this.ERROR_CLASS_NAME;
        }

        if (this._shouldCheckLift(item) && item.isNoLift) {
            return this.ERROR_CLASS_NAME;
        }

        if (this._shouldCheckExterior(item) && item.isInterior) {
            return this.ERROR_CLASS_NAME;
        }
        return this.SUCCESS_CLASS_NAME;
    }

    static createInformation(item) {
        let html = `<div class='${this.CONTAINER_CLASS_NAME} ${this.ITEM_STATE_CLASS_NAME(item)}'>`;
        html += `<div class='${this.INFORMATION_CONTAINER_CLASS_NAME}'>`;
        html += this._createPercentagesPriceHTML(item);
        html += this._createPriceHTML(item);
        html += this._createPriceMeterHTML(item);
        html += `</div>`;

        html += `<div class='${this.INFORMATION_CONTAINER_CLASS_NAME}'>`;
        html += this._createGarageHTML(item);
        html += this._createLiftHTML(item);
        html += this._createExteriorHTML(item);
        html += `</div>`;

        html += `<div class='${this.INFORMATION_CONTAINER_CLASS_NAME}'>`;
        html += this._createLocationName(item);

        if (item.location && !item.location.errorMessage) {
            html += this._createLocationTrain(item);
            html += this._createLocationBus(item);
            html += this._createLocationGym(item);
            html += this._createLocationPool(item);
            html += this._createLocationSupermarket(item);
            html += this._createLocationSmoke(item);
            html += this._createLocationPharmacy(item);
        } else if (item.location && item.location.errorMessage) {
            html += this._createError(item.location.errorMessage);
        } else {
            html += this._createNeutral(`Cargando datos...`);
        }

        html += `</div>`;
        html += `</div>`;

        return html;
    }

    static _createPercentagesPriceHTML(item) {
        if (!Preferences.get('percentages')) return ``;

        let html = `<div class='${this.INFORMATION_CLASS_NAME}'>`;
        if (Preferences.get('percentages_20')) {
            html += this._createPercentagePriceHTML(item, 20);
        }

        if (Preferences.get('percentages_30')) {
            html += this._createPercentagePriceHTML(item, 30);
        }

        if (Preferences.get('percentages_50')) {
            html += this._createPercentagePriceHTML(item, 50);
        }
        html += `</div>`;

        return html;
    }

    static _createPercentagePriceHTML(item, percent) {
        const calculation = Math.round(item.price * percent / 100);
        const currentMoney = Preferences.get('currentMoney'); 
        const percentText = `${percent}%`;
        if (currentMoney === 0) {
            return this._createIndividual(percentText, calculation);
        }
        
        if (currentMoney >= calculation) {
            return this._createTextSuccess(percentText, calculation);
        }
        return this._createTextError(percentText, calculation);
    }

    static _createPriceHTML(item) {
        const desiredPrice = Preferences.get('max-price');
        if (!desiredPrice) return ``;

        const desiredTwentyFivePercentMore = Math.round(desiredPrice * 1.25);

        if (item.isProperPrice(desiredPrice)) {
            return this._createSuccess('Precio');
        } else if (item.isProperPrice(desiredTwentyFivePercentMore)) {
            return this._createWarning('Precio');
        }
        return this._createError('Precio');
    }

    static _createPriceMeterHTML(item) {
        const desiredPricePerMeter = Preferences.get('max-price-per-meter');
        if (!desiredPricePerMeter) return this._createIndividual('m²', item.priceMeter);

        const desiredTwentyFivePercentMore = Math.round(desiredPricePerMeter * 1.25);

        if (item.priceMeter <= desiredPricePerMeter) {
            return this._createTextSuccess('m²', item.priceMeter)
        } else if (item.priceMeter <= desiredTwentyFivePercentMore) {
            return this._createTextWarning('m²', item.priceMeter)
        }
        return this._createTextError('m²', item.priceMeter)
    }

    static _createGarageHTML(item) {
        if (!this._shouldCheckGarage()) {
            return ``;
        }

        if (item.hasGarage) {
            return this._createSuccess('Garaje');
        }
        return this._createError('Garaje');
    }

    static _shouldCheckGarage() {
        return Preferences.get('garage');
    }

    static _createLiftHTML(item) {
        if (!this._shouldCheckLift(item)) {
            return ``;
        }

        if (item.isNoLift) {
            return this._createError('Ascensor');
        } else if (item.hasLift) {
            return this._createSuccess('Ascensor');
        }
        return this._createMissing('Ascensor');
    }

    static _shouldCheckLift(item) {
        return Preferences.get('lift')
            && !item.isHouse()
            && !item.isGround();
    }

    static _createExteriorHTML(item) {
        if (!this._shouldCheckExterior(item)) {
            return ``;
        }

        if (item.isExterior) {
            return this._createSuccess('Exterior');
        }

        if (item.isInterior) {
            return this._createError('Exterior');
        }
        return this._createMissing('Exterior');
    }

    static _createLocationName(item) {
        let html = `<div class='information'>`
        if (item.location?.name) {
            html += this._createNeutral(item.location.name);
        } else {
            html += this._createNeutral(item.locationName);
        }
        if (item.location?.altitude) html += this._createIndividual('Altitud (m):', item.location.altitude);

        html += `</div>`
        return html;
    }

    static _createLocationTrain(item) {
        if (!Preferences.get('train')) return ``;

        if (item.location?.train) {
            return this._createSuccess('Tren');
        }
        return this._createError('Tren');
    }

    static _createLocationBus(item) {
        if (!Preferences.get('bus')) return ``;

        if (item.location?.bus) {
            return this._createSuccess('Bus');
        }
        return this._createError('Bus');
    }

    static _createLocationGym(item) {
        if (!Preferences.get('gym')) return ``;

        if (item.location?.gym) {
            return this._createSuccess('Gimnasio');
        }
        return this._createError('Gimnasio');
    }

    static _createLocationPool(item) {
        if (!Preferences.get('pool')) return ``;

        if (item.location?.pool) {
            return this._createSuccess('Piscina');
        }
        return this._createError('Piscina');
    }

    static _createLocationSupermarket(item) {
        if (!Preferences.get('supermarket')) return ``;

        let html = `<div>`;
        if (item.location?.supermarkets.length) {
            html += this._createSuccess(`${item.location.supermarkets.length} supermercados`);
        } else {
            html += this._createError('Supermercado');
        }
        html += `</div>`;

        return html;
    }

    static _createLocationSmoke(item) {
        if (!Preferences.get('smoke')) return ``;

        if (item.location?.smoke) {
            return this._createSuccess('Estanco');
        }
        return this._createError('Estanco');
    }

    static _createLocationPharmacy(item) {
        if (!Preferences.get('pharmacy')) return ``;

        if (item.location?.pharmacy) {
            return this._createSuccess('Farmacia');
        }
        return this._createError('Farmacia');
    }

    static _shouldCheckExterior(item) {
        return Preferences.get('exterior')
            && !item.isHouse();
    }

    static _createSuccess(infoText) {
        return this._createTextSuccess(IconSvg.TICK, infoText);
    }

    static _createTextSuccess(strongText, infoText) {
        return this._createIndividual(strongText, infoText, this.SUCCESS_CLASS_NAME);
    }

    static _createWarning(infoText) {
        return this._createTextWarning('⚠️', infoText);
    }

    static _createMissing(infoText) {
        return this._createTextWarning('?', infoText);
    }

    static _createTextWarning(strongText, infoText) {
        return this._createIndividual(strongText, infoText, this.WARNING_CLASS_NAME);
    }

    static _createError(infoText) {
        return this._createTextError('🚫', infoText);
    }

    static _createTextError(strongText, infoText) {
        return this._createIndividual(strongText, infoText, this.ERROR_CLASS_NAME);
    }

    static _createNeutral(infoText) {
        return this._createIndividual('ⓘ', infoText)
    }

    static _createIndividual(strongText, infoText, className = '') {
        return `<span class='${className}'><strong>${strongText}</strong> ${infoText}</span>`;
    }

}